@@ -2,15 +2,14 @@
 import collections
 import errno
 import logging
-import os
 import signal
-import sys
 import tokenize
+from typing import Dict, List, Optional, Tuple
 
 try:
     import multiprocessing
 except ImportError:
-    multiprocessing = None
+    multiprocessing = None  # type: ignore
 
 from flake8 import defaults
 from flake8 import exceptions
@@ -73,9 +72,7 @@ class Manager(object):
         self.options = style_guide.options
         self.checks = checker_plugins
         self.jobs = self._job_count()
-        self.using_multiprocessing = self.jobs > 1
-        self.processes = []
-        self.checkers = []
+        self.checkers = []  # type: List[FileChecker]
         self.statistics = {
             "files": 0,
             "logical lines": 0,
@@ -188,23 +185,15 @@ class Manager(object):
                 return False
             path = self.options.stdin_display_name
 
-        exclude = self.options.exclude
-        if not exclude:
-            return False
-        basename = os.path.basename(path)
-        if utils.fnmatch(basename, exclude):
-            LOG.debug('"%s" has been excluded', basename)
-            return True
-
-        absolute_path = os.path.abspath(path)
-        match = utils.fnmatch(absolute_path, exclude)
-        LOG.debug(
-            '"%s" has %sbeen excluded', absolute_path, "" if match else "not "
+        return utils.matches_filename(
+            path,
+            patterns=self.options.exclude,
+            log_message='"%(path)s" has %(whether)sbeen excluded',
+            logger=LOG,
         )
-        return match
 
     def make_checkers(self, paths=None):
-        # type: (List[str]) -> NoneType
+        # type: (Optional[List[str]]) -> None
         """Create checkers for each file."""
         if paths is None:
             paths = self.arguments
@@ -255,7 +244,7 @@ class Manager(object):
         LOG.info("Checking %d files", len(self.checkers))
 
     def report(self):
-        # type: () -> (int, int)
+        # type: () -> Tuple[int, int]
         """Report all of the errors found in the managed file checkers.
 
         This iterates over each of the checkers and reports the errors sorted
@@ -279,18 +268,20 @@ class Manager(object):
 
     def run_parallel(self):
         """Run the checkers in parallel."""
-        final_results = collections.defaultdict(list)
-        final_statistics = collections.defaultdict(dict)
+        # fmt: off
+        final_results = collections.defaultdict(list)  # type: Dict[str, List[Tuple[str, int, int, str, Optional[str]]]]  # noqa: E501
+        final_statistics = collections.defaultdict(dict)  # type: Dict[str, Dict[str, None]]  # noqa: E501
+        # fmt: on
 
         try:
             pool = multiprocessing.Pool(self.jobs, _pool_init)
         except OSError as oserr:
             if oserr.errno not in SERIAL_RETRY_ERRNOS:
                 raise
-            self.using_multiprocessing = False
             self.run_serial()
             return
 
+        pool_closed = False
         try:
             pool_map = pool.imap_unordered(
                 _run_checks,
@@ -305,9 +296,9 @@ class Manager(object):
                 final_statistics[filename] = statistics
             pool.close()
             pool.join()
-            pool = None
+            pool_closed = True
         finally:
-            if pool is not None:
+            if not pool_closed:
                 pool.terminate()
                 pool.join()
 
@@ -334,16 +325,10 @@ class Manager(object):
         fallback to serial processing.
         """
         try:
-            if self.using_multiprocessing:
+            if self.jobs > 1 and len(self.checkers) > 1:
                 self.run_parallel()
             else:
                 self.run_serial()
-        except OSError as oserr:
-            if oserr.errno not in SERIAL_RETRY_ERRNOS:
-                LOG.exception(oserr)
-                raise
-            LOG.warning("Running in serial after OS exception, %r", oserr)
-            self.run_serial()
         except KeyboardInterrupt:
             LOG.warning("Flake8 was interrupted by the user")
             raise exceptions.EarlyQuit("Early quit while running checks")
@@ -361,9 +346,6 @@ class Manager(object):
     def stop(self):
         """Stop checking files."""
         self._process_statistics()
-        for proc in self.processes:
-            LOG.info("Joining %s to the main process", proc.name)
-            proc.join()
 
 
 class FileChecker(object):
@@ -386,7 +368,9 @@ class FileChecker(object):
         self.options = options
         self.filename = filename
         self.checks = checks
-        self.results = []
+        # fmt: off
+        self.results = []  # type: List[Tuple[str, int, int, str, Optional[str]]]  # noqa: E501
+        # fmt: on
         self.statistics = {
             "tokens": 0,
             "logical lines": 0,
@@ -405,22 +389,22 @@ class FileChecker(object):
         return "FileChecker for {}".format(self.filename)
 
     def _make_processor(self):
+        # type: () -> Optional[processor.FileProcessor]
         try:
             return processor.FileProcessor(self.filename, self.options)
-        except IOError:
+        except IOError as e:
             # If we can not read the file due to an IOError (e.g., the file
             # does not exist or we do not have the permissions to open it)
             # then we need to format that exception for the user.
             # NOTE(sigmavirus24): Historically, pep8 has always reported this
             # as an E902. We probably *want* a better error code for this
             # going forward.
-            (exc_type, exception) = sys.exc_info()[:2]
-            message = "{0}: {1}".format(exc_type.__name__, exception)
+            message = "{0}: {1}".format(type(e).__name__, e)
             self.report("E902", 0, 0, message)
             return None
 
     def report(self, error_code, line_number, column, text, line=None):
-        # type: (str, int, int, str) -> str
+        # type: (str, int, int, str, Optional[str]) -> str
         """Report an error by storing it in the results list."""
         if error_code is None:
             error_code, text = text.split(" ", 1)
@@ -451,10 +435,12 @@ class FileChecker(object):
             return plugin["plugin"](**arguments)
         except Exception as all_exc:
             LOG.critical(
-                "Plugin %s raised an unexpected exception", plugin["name"]
+                "Plugin %s raised an unexpected exception",
+                plugin["name"],
+                exc_info=True,
             )
             raise exceptions.PluginExecutionFailed(
-                plugin=plugin, excetion=all_exc
+                plugin=plugin, exception=all_exc
             )
 
     @staticmethod
@@ -462,7 +448,7 @@ class FileChecker(object):
         token = ()
         if len(exception.args) > 1:
             token = exception.args[1]
-            if len(token) > 2:
+            if token and len(token) > 2:
                 row, column = token[1:3]
         else:
             row, column = (1, 0)
@@ -498,14 +484,10 @@ class FileChecker(object):
         """Run all checks expecting an abstract syntax tree."""
         try:
             ast = self.processor.build_ast()
-        except (ValueError, SyntaxError, TypeError):
-            (exc_type, exception) = sys.exc_info()[:2]
-            row, column = self._extract_syntax_information(exception)
+        except (ValueError, SyntaxError, TypeError) as e:
+            row, column = self._extract_syntax_information(e)
             self.report(
-                "E999",
-                row,
-                column,
-                "%s: %s" % (exc_type.__name__, exception.args[0]),
+                "E999", row, column, "%s: %s" % (type(e).__name__, e.args[0])
             )
             return
 
@@ -550,21 +532,38 @@ class FileChecker(object):
         self.processor.next_logical_line()
 
     def run_physical_checks(self, physical_line, override_error_line=None):
-        """Run all checks for a given physical line."""
+        """Run all checks for a given physical line.
+
+        A single physical check may return multiple errors.
+        """
         for plugin in self.checks["physical_line_plugins"]:
             self.processor.update_checker_state_for(plugin)
             result = self.run_check(plugin, physical_line=physical_line)
-            if result is not None:
-                column_offset, text = result
-                error_code = self.report(
-                    error_code=None,
-                    line_number=self.processor.line_number,
-                    column=column_offset,
-                    text=text,
-                    line=(override_error_line or physical_line),
-                )
 
-                self.processor.check_physical_error(error_code, physical_line)
+            if result is not None:
+                # This is a single result if first element is an int
+                column_offset = None
+                try:
+                    column_offset = result[0]
+                except (IndexError, TypeError):
+                    pass
+
+                if isinstance(column_offset, int):
+                    # If we only have a single result, convert to a collection
+                    result = (result,)
+
+                for result_single in result:
+                    column_offset, text = result_single
+                    error_code = self.report(
+                        error_code=None,
+                        line_number=self.processor.line_number,
+                        column=column_offset,
+                        text=text,
+                        line=(override_error_line or physical_line),
+                    )
+                    self.processor.check_physical_error(
+                        error_code, physical_line
+                    )
 
     def process_tokens(self):
         """Process tokens and trigger checks.
@@ -586,11 +585,6 @@ class FileChecker(object):
             elif parens == 0:
                 if processor.token_is_newline(token):
                     self.handle_newline(token_type)
-                elif (
-                    processor.token_is_comment(token)
-                    and len(file_processor.tokens) == 1
-                ):
-                    self.handle_comment(token, text)
 
         if file_processor.tokens:
             # If any tokens are left over, process them
@@ -601,6 +595,7 @@ class FileChecker(object):
         """Run checks against the file."""
         try:
             self.process_tokens()
+            self.run_ast_checks()
         except exceptions.InvalidSyntax as exc:
             self.report(
                 exc.error_code,
@@ -609,21 +604,10 @@ class FileChecker(object):
                 exc.error_message,
             )
 
-        self.run_ast_checks()
-
         logical_lines = self.processor.statistics["logical lines"]
         self.statistics["logical lines"] = logical_lines
         return self.filename, self.results, self.statistics
 
-    def handle_comment(self, token, token_text):
-        """Handle the logic when encountering a comment token."""
-        # The comment also ends a physical line
-        token = list(token)
-        token[1] = token_text.rstrip("\r\n")
-        token[3] = (token[2][0], token[2][1] + len(token[1]))
-        self.processor.tokens = [tuple(token)]
-        self.run_logical_checks()
-
     def handle_newline(self, token_type):
         """Handle the logic when encountering a newline token."""
         if token_type == tokenize.NEWLINE:
