@@ -1,10 +1,10 @@
 """Tests for the Application class."""
 import optparse
+import sys
 
 import mock
 import pytest
 
-from flake8 import exceptions
 from flake8.main import application as app
 
 
@@ -61,14 +61,6 @@ def test_exit_does_raise(result_count, catastrophic, exit_zero, value,
     assert excinfo.value.args[0] is value
 
 
-def test_missing_default_formatter(application):
-    """Verify we raise an ExecutionError when there's no default formatter."""
-    application.formatting_plugins = {}
-
-    with pytest.raises(exceptions.ExecutionError):
-        application.formatter_for('fake-plugin-name')
-
-
 def test_warns_on_unknown_formatter_plugin_name(application):
     """Verify we log a warning with an unfound plugin."""
     default = mock.Mock()
@@ -106,3 +98,12 @@ def test_prelim_opts_args(application):
     assert application.prelim_opts.statistics
     assert application.prelim_opts.verbose
     assert application.prelim_args == ['src', 'setup.py']
+
+
+def test_prelim_opts_handles_empty(application):
+    """Verify empty argv lists are handled correctly."""
+    irrelevant_args = ['myexe', '/path/to/foo']
+    with mock.patch.object(sys, 'argv', irrelevant_args):
+        application.parse_preliminary_options_and_args([])
+
+        assert application.prelim_args == []
